==> json-joy-7.11.0.tgz: package/esm/util/codegen/Codegen.js <==
import { compileClosure } from '.';
export class CodegenStepExecJs {
    js;
    constructor(js) {
        this.js = js;
    }
}
export class Codegen {
    steps = [];
    options;
    constructor(opts) {
        this.options = {
            args: 'r0',
            name: '',
            prologue: '',
            epilogue: '',
            processSteps: (steps) => steps.filter((step) => step instanceof CodegenStepExecJs),
            linkable: {},
            ...opts,
        };
    }
    js(js) {
        this.steps.push(new CodegenStepExecJs(js));
    }
    step(step) {
        this.steps.push(step);
    }
    registerCounter = 0;
    getRegister() {
        return `r${this.registerCounter++}`;
    }
    dependencies = [];
    dependencyNames = [];
    linkDependency(dep, name = 'd' + this.dependencies.length) {
        this.dependencies.push(dep);
        this.dependencyNames.push(name);
        return name;
    }
    linkDependencies(deps) {
        return deps.map((dep) => this.linkDependency(dep));
    }
    linked = {};
    link(name) {
        if (this.linked[name])
            return;
        this.linked[name] = 1;
        this.linkDependency(this.options.linkable[name], name);
    }
    constants = [];
    constantNames = [];
    addConstant(constant, name = 'c' + this.constants.length) {
        this.constants.push(constant);
        this.constantNames.push(name);
        return name;
    }
    addConstants(constants) {
        return constants.map((constant) => this.addConstant(constant));
    }
    generate(opts = {}) {
        const { name, args, prologue, epilogue } = { ...this.options, ...opts };
        const steps = this.options.processSteps(this.steps);
        const js = `(function(${this.dependencyNames.join(', ')}) {
${this.constants.map((constant, index) => `var ${this.constantNames[index]} = (${constant});`).join('\n')}
return ${name ? `function ${name}` : 'function'}(${args}){
${prologue}
${steps.map((step) => step.js).join('\n')}
${epilogue}
}})`;
        return {
            deps: this.dependencies,
            js: js,
        };
    }
    compile(opts) {
        return compileClosure(this.generate(opts));
    }
}

==> json-joy-8.0.1.tgz: package/esm/util/codegen/Codegen.js <==
import { compileClosure } from '.';
export class CodegenStepExecJs {
    js;
    constructor(js) {
        this.js = js;
    }
}
export class Codegen {
    steps = [];
    options;
    constructor(opts) {
        this.options = {
            args: ['r0'],
            name: '',
            prologue: '',
            epilogue: '',
            processSteps: (steps) => steps.filter((step) => step instanceof CodegenStepExecJs),
            linkable: {},
            ...opts,
        };
        this.registerCounter = this.options.args.length;
    }
    js(js) {
        this.steps.push(new CodegenStepExecJs(js));
    }
    step(step) {
        this.steps.push(step);
    }
    registerCounter;
    getRegister() {
        return `r${this.registerCounter++}`;
    }
    r() {
        return this.getRegister();
    }
    dependencies = [];
    dependencyNames = [];
    linkDependency(dep, name = 'd' + this.dependencies.length) {
        this.dependencies.push(dep);
        this.dependencyNames.push(name);
        return name;
    }
    linkDependencies(deps) {
        return deps.map((dep) => this.linkDependency(dep));
    }
    linked = {};
    link(name) {
        if (this.linked[name])
            return;
        this.linked[name] = 1;
        this.linkDependency(this.options.linkable[name], name);
    }
    constants = [];
    constantNames = [];
    addConstant(constant, name = 'c' + this.constants.length) {
        this.constants.push(constant);
        this.constantNames.push(name);
        return name;
    }
    addConstants(constants) {
        return constants.map((constant) => this.addConstant(constant));
    }
    generate(opts = {}) {
        const { name, args, prologue, epilogue } = { ...this.options, ...opts };
        const steps = this.options.processSteps(this.steps);
        const js = `(function(${this.dependencyNames.join(', ')}) {
${this.constants.map((constant, index) => `var ${this.constantNames[index]} = (${constant});`).join('\n')}
return ${name ? `function ${name}` : 'function'}(${args.join(',')}){
${prologue}
${steps.map((step) => step.js).join('\n')}
${epilogue}
}})`;
        return {
            deps: this.dependencies,
            js: js,
        };
    }
    compile(opts) {
        const closure = this.generate(opts);
        return compileClosure(closure);
    }
}
